unit PropInspect;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, System.Rtti,
  JvComponentBase, JvInspector, JvExControls;

type
  ReadableAttribute = class(TCustomAttribute)
  private
    FDisplay: string;
    FDescription: string;
  public
    constructor Create(const ADisplay: string); overload;
    constructor Create(const ADisplay, ADescription: string); overload;
    property Display: string read FDisplay;
    property Description: string read FDescription;
  end;

  TfrmPropInspect = class(TForm)
    Splitter1: TSplitter;
    Memo1: TMemo;
    Panel1: TPanel;
    Button1: TButton;
    procedure JvInspector1AfterItemCreate(Sender: TObject; Item: TJvCustomInspectorItem);
    procedure FormCreate(Sender: TObject);
    procedure JvInspector1ItemSelected(Sender: TObject);
  private
    FCtx: TRttiContext;
  public
    { Public declarations }
    JvInspector: TJvInspector;
    JvInspectorDotNETPainter: TJvInspectorDotNETPainter;
    class function Execute(AObject: TObject): Boolean; static;
  end;

implementation

{$R *.dfm}

{ ReadableAttribute }

constructor ReadableAttribute.Create(const ADisplay: string);
begin
  Create(ADisplay, '');
end;

constructor ReadableAttribute.Create(const ADisplay, ADescription: string);
begin
  inherited Create;
  FDisplay := ADisplay;
  FDescription := ADescription;
end;

{ TfrmPropInspect }

procedure TfrmPropInspect.FormCreate(Sender: TObject);
begin
  FCtx := TRttiContext.Create;
  JvInspector := TJvInspector.Create(self);
  JvInspectorDotNETPainter := TJvInspectorDotNETPainter.Create(self);

  JvInspector.Parent := Self;
  JvInspector.Divider := 220;
  JvInspector.ItemHeight := 19;
  JvInspector.Style := isItemPainter;
  JvInspector.Align := alClient;
  JvInspector.TabStop := True;
  JvInspector.Painter := JvInspectorDotNETPainter;
  JvInspector.AfterItemCreate := JvInspector1AfterItemCreate;
  JvInspector.OnItemSelected := JvInspector1ItemSelected;
end;

procedure TfrmPropInspect.JvInspector1AfterItemCreate(Sender: TObject; Item: TJvCustomInspectorItem);
var
  LObj: TObject;
  LAttr: ReadableAttribute;
begin
  if (Item.Data <> nil) and (Item.Data is TJvInspectorPropData) then
  begin
    LObj := TJvInspectorPropData(Item.Data).Instance;
    LAttr := FCtx.GetType(LObj.ClassType).GetProperty(Item.Name).GetAttribute<ReadableAttribute>;
    if (LAttr <> nil) and (LAttr.Display <> '') then
      Item.DisplayName := LAttr.Display;
  end;
end;

procedure TfrmPropInspect.JvInspector1ItemSelected(Sender: TObject);
var
  LItem: TJvCustomInspectorItem;
  LObj: TObject;
  LAttr: ReadableAttribute;
begin
  Memo1.Clear;
  LItem := JvInspector.Selected;
  if (LItem <> nil) and (LItem.Data <> nil) and (LItem.Data is TJvInspectorPropData) then
  begin
    LObj := TJvInspectorPropData(LItem.Data).Instance;
    LAttr := FCtx.GetType(LObj.ClassType).GetProperty(LItem.Name).GetAttribute<ReadableAttribute>;
    if LAttr <> nil then
      Memo1.Text := LAttr.Description;
  end;
end;

class function TfrmPropInspect.Execute(AObject: TObject): Boolean;
var
  LDlg: TfrmPropInspect;
begin
  LDlg := TfrmPropInspect.Create(nil);
  try
    LDlg.Caption := 'Inspect: ' + AObject.ClassName;
    LDlg.JvInspector.AddComponent(AObject);
    Result := LDlg.ShowModal = mrOk;
  finally
    LDlg.Free;
  end;
end;

end.
